Imports System.Drawing
Imports System.Drawing.Imaging
Public Class BitmapFast

#Region "private"
    Private b() As Byte
    Private i_Width As Integer
    Private i_Height As Integer    
    Private i_Stride As Integer
    Private i_ColorSize As Integer
#End Region

#Region "public"
    Public Disposed As Boolean
#End Region

#Region "New"
    Public Sub New(ByVal filename As String)
        If (filename = "") Or (System.IO.File.Exists(filename) = False) Then
            MsgBox("file '" & filename & "' not found!")
            Exit Sub
        End If

        Dim pv_bitmap As Bitmap = Nothing
        pv_bitmap = New Bitmap(filename)
        i_Width = pv_bitmap.Width
        i_Height = pv_bitmap.Height

        Dim bounds As Rectangle = New Rectangle(0, 0, i_Width, i_Height)
        Dim bitmapData As BitmapData = pv_bitmap.LockBits(bounds, ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb)        
        i_stride = bitmapData.Stride
        i_ColorSize = bitmapData.Stride / bitmapData.Width
        ReDim b(bitmapData.Height * i_stride)
        System.Runtime.InteropServices.Marshal.Copy(bitmapData.Scan0, b, 0, b.Length - 1)
        pv_bitmap.UnlockBits(bitmapData)

        pv_bitmap.Dispose()
        pv_bitmap = Nothing
    End Sub
#End Region

#Region "Function"
    Public Function Getpixel(ByVal x As Integer, ByVal y As Integer) As Color

        Dim red, green, blue As Byte        
        red = b(x * i_ColorSize + y * i_Stride)
        green = b((x * i_ColorSize + 1) + y * i_Stride)
        blue = b((x * i_ColorSize + 2) + y * i_Stride)

        Return Color.FromArgb(255, red, green, blue)
    End Function

    Public Sub Setpixel(ByVal x As Integer, ByVal y As Integer, ByVal color As Color)        
        b(x * i_ColorSize + y * i_Stride) = color.R
        b((x * i_ColorSize + 1) + y * i_Stride) = color.G
        b((x * i_ColorSize + 2) + y * i_Stride) = color.B
    End Sub
#End Region

#Region "Save"
    Public Sub Save(ByVal filename As String, ByVal myFormat As ImageFormat)
        Dim pv_bitmap As Bitmap = Nothing
        pv_bitmap = New Bitmap(i_Width, i_Height, PixelFormat.Format24bppRgb)

        Dim bounds As Rectangle = New Rectangle(0, 0, i_Width, i_Height)
        Dim bitmapData As Imaging.BitmapData = pv_bitmap.LockBits(bounds, ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb)

        System.Runtime.InteropServices.Marshal.Copy(b, 0, bitmapData.Scan0, b.Length - 1)
        pv_bitmap.Save(filename, myFormat)
        pv_bitmap.UnlockBits(bitmapData)

        pv_bitmap.Dispose()
        pv_bitmap = Nothing

    End Sub

#End Region

#Region "GetBitmap"
    Public Function Get_Bitmap() As Bitmap

        Get_Bitmap = New Bitmap(i_Width, i_Height, PixelFormat.Format24bppRgb)

        Dim bounds As Rectangle = New Rectangle(0, 0, i_Width, i_Height)
        Dim bitmapData As Imaging.BitmapData = Get_Bitmap.LockBits(bounds, ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb)
        bitmapData.Stride = CInt(bitmapData.Stride / i_Width) * i_Width
        System.Runtime.InteropServices.Marshal.Copy(b, 0, bitmapData.Scan0, b.Length - 1)
        Get_Bitmap.UnlockBits(bitmapData)

    End Function
#End Region

#Region "property"
    Public ReadOnly Property Width() As Integer
        Get
            Return i_Width
        End Get
    End Property

    Public ReadOnly Property Height() As Integer
        Get
            Return i_Height
        End Get
    End Property
#End Region

#Region "dispose"
    Public Sub Dispose()
        If Disposed = True Then Exit Sub
        Erase b
        Disposed = True
    End Sub

    Protected Overrides Sub Finalize()
        MyBase.Finalize()
        If Disposed = False Then Dispose()
    End Sub
#End Region

End Class
